#ifndef WIN32_AUDIO_H
#define WIN32_AUDIO_H

#ifdef ENABLE_DIRECTMUSIC
  #include <dmusici.h>
  #include <dmusicc.h>
#endif

#include <stdio.h>
#include "types.h"
#include "fmod.h"

extern bool SetGlobalSoundVolume(int volume);
extern int  GetGlobalSoundVolume();
extern bool SetGlobalMusicVolume(int volume);
extern int  GetGlobalMusicVolume();

extern unsigned long AudioOpenFileCallback(char* filename);
extern void AudioCloseFileCallback(unsigned long id);
extern long AudioReadFileCallback(void* buffer, long size, unsigned long id);
extern void AudioSeekFileCallback(unsigned long id, long offset, signed char mode);
extern long AudioTellFileCallback(unsigned long id);



enum MusicType { MUSIC_MP3, MUSIC_WAV, MUSIC_MOD, MUSIC_MIDI };

class CSoundEffect
{
public:
  CSoundEffect();
  ~CSoundEffect();

  bool Load(const char* filename);
  void Destroy();
  void Play();
  void PlayAbsolute(int volume);
  void Stop();
  unsigned long GetID();
  void SetID(unsigned long id);
  FILE* GetFile();
  void SetFile(FILE* file);
  long GetFileSize();
  void SetFileSize(long size);
  long GetFileStartPosition();
  void SetFileStartPosition(long start);

private:
  long m_Size;
  long m_FileStartPosition;
  long m_Channel;
  bool m_Loaded;
  
  #ifdef ENABLE_DIRECTMUSIC
  IDirectMusic            *directmusic;
  IDirectMusicLoader      *directmusicloader;
  IDirectMusicSegment     *directmusicsegment;
  IDirectMusicSegmentState *directmusicsegmentstate;
  IDirectMusicPerformance *directmusicperformance;
  #endif
  FSOUND_SAMPLE* sample;
  unsigned long fhandle;
  FILE* fpointer;
};

class CMusicModule
{
public:
  CMusicModule();
  ~CMusicModule();

  bool Load(const char* filename);
  void Destroy();
  void Play();
  void Stop();
  void SetVolume(int volume);
  unsigned long GetID();
  void SetID(unsigned long id);
  FILE* GetFile();
  void SetFile(FILE* file);
  long GetFileSize();
  void SetFileSize(long size);
  long GetFileStartPosition();
  void SetFileStartPosition(long start);

private:
  MusicType m_MusicType;
  long m_Size;
  long m_FileStartPosition;
  long m_StreamChannel;
  bool m_IsPlaying;
  bool m_Loaded;

  FSOUND_STREAM* stream;
  FMUSIC_MODULE* module;
  unsigned long fhandle;
  FILE* fpointer;
};

#endif
